/*
AC '97 Audio PCI Configuration Space (D31:F5)
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Offset		Mnemonic 	Register 					Default		Access
00-01h		VID 		Vendor Identification 				8086h 		RO
02-03h		DID 		Device Identification 				24C5h 		RO
04-05h		PCICMD 		PCI Command 					0000 		R/W, RO
06-07h		PCISTS 		PCI Device Status 				0280h 		R/WC, RO
08h		RID 		Revision Identification 			See Note 	RO
09h		PI 		Programming Interface 				00 		RO
0Ah		SCC 		Sub Class Code 					01h 		RO
0Bh		BCC 		Base Class Code 				04h 		RO
0Eh 		HEDT 		Header Type 					00 		RO
10-13h		NAMBAR 		Native Audio Mixer Base Address 		00000001h 	R/W, RO
14-17h		NABMBAR 	Native Audio Bus Mastering Base Address 	00000001h 	R/W, RO
18-1Bh		MMBAR		Mixer Base Address(Mem) 			00000000h	R/W, RO
1C-1Fh		MBBAR		Bus Master Base Address(Mem) 			00000000h	R/W, RO
2C-2Dh		SVID		Subsystem Vendor ID				0000h		R/WO
2E-2Fh		SID		Subsystem ID					0000h		R/WO
34h		CAP_PTR		Capabilities Pointer				50h		RO
3Ch		INTR_LN		Interrupt Line					00h		R/W
3Dh		INTR_PN		Interrupt Pin					02h		RO
40h		PCID		Programmable Codec ID				01h		R/W
41h		CFG		Configuration					00h		R/W
5051h	PID		PCI Power Management ID				0001h		RO
5253h	PC 		PC -Power Management Capabilities		C9C2h		RO
5455h	PCS 		Power Management Control and Status		0000h		R/W, R/WC
*/

#define AC97_PCI_VID		0x00                                            
#define AC97_PCI_DID 		0x02                                           
#define AC97_PCI_PCICMD		0x04                                         
#define AC97_PCI_PCIST		0x06                                          
#define AC97_PCI_RID 		0x08                                           
#define AC97_PCI_PI		0x09                                            
#define AC97_PCI_SCC		0x0A                                            
#define AC97_PCI_BCC		0x0B                                            
#define AC97_PCI_HEDT 		0x0E                                          
#define AC97_PCI_NAMBAR		0x10                                         
#define AC97_PCI_NABMBAR	0x14                                         
#define AC97_PCI_MMBAR		0x18                                          
#define AC97_PCI_MBBAR		0x1C                                          
#define AC97_PCI_SVID		0x2C                                           
#define AC97_PCI_SID		0x2E                                            
#define AC97_PCI_CAP_PTR	0x34                                         
#define AC97_PCI_INTR_LN	0x3C                                         
#define AC97_PCI_INTR_PN	0x3D                                         
#define AC97_PCI_PCID		0x40                                           
#define AC97_PCI_CFG		0x41                                            
#define AC97_PCI_PID		0x50                                            
#define AC97_PCI_PC 		0x52                                            
#define AC97_PCI_PCS 		0x54

/*

AC 97 Audio I/O Space (D31:F5)
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Primary Offset	Secondary Offset	Tertiary Offset		NAMBAR Exposed Registers
(Codec ID =00)	(Codec ID =01) 		(Codec ID =10) 		(D31:F5)

ID00	ID01	ID10		NAMBAR Exposed Registers (D31:F5)
00h	80h	100h  		Reset
02h	82h	102h  		Master Volume
04h	84h	104h  		Aux Out Volume
06h	86h	106h  		Mono Volume
08h 	88h	108h  		Master Tone (R & L)
0Ah	8Ah	10Ah  		PC_BEEP Volume
0Ch	8Ch	10Ch  		Phone Volume
0Eh	8Eh	10Eh		Mic Volume
10h	90h	110h		Line In Volume
12h	92h	112h		CD Volume
14h	94h	114h		Video Volume
16h	96h	116h		Aux In Volume
18h	98h	118h		PCM Out Volume
1Ah	9Ah	11Ah		Record Select
1Ch	9Ch	11Ch		Record Gain
1Eh	9Eh	11Eh		Record Gain Mic
20h	A0h	120h		General Purpose
22h	A2h	122h		3D Control
24h	A4h	124h		AC '97 RESERVED
26h	A6h	126h		Powerdown Ctrl/Stat
28h	A8h	128h		Extended Audio
2Ah	AAh	12Ah		Extended Audio Ctrl/Stat
2Ch	ACh	12Ch		PCM Front DAC Rate
2Eh	AEh	12Eh		PCM Surround DAC Rate
30h	B0h	130h		PCM LFE DAC Rate
32h	B2h	132h		PCM LR ADC Rate
34h	B4h	134h		MIC ADC Rate
36h	B6h	136h		6Ch Vol: C, LFE
*/

/* 	AC'97 codec registers */
#define AC97_RESET			0x00	/* Reset */
#define AC97_MASTER_VOLUME		0x02	/* Master Volume */
#define AC97_AUX_OUT_VOLUME		0x04	/* Aux Out Volume */
#define AC97_MONO_VOLUME		0x06	/* Mono Volume */
#define AC97_MASTER_TONE		0x08	/* Master Tone (R & L) */ /* high byte = bass, low byte = treble */
#define AC97_PC_BEEP_VOLUME 		0x0A	/* PC_BEEP Volume */
#define AC97_PHONE_VOLUME		0x0C	/* Phone Volume */
#define AC97_MIC_VOLUME			0x0E	/* Mic Volume */
#define AC97_LINE_IN_VOLUME		0x10	/* Line In Volume */
#define AC97_CD_VOLUME			0x12	/* CD Volume */
#define AC97_VIDEO_VOLUME		0x14	/* Video Volume */
#define AC97_AUX_IN_VOLUME		0x16	/* Aux In Volume */
#define AC97_PCM_OUT_VOLUME		0x18	/* PCM Out Volume */
#define AC97_RECORD_SELECT		0x1A	/* Record Select */
#define AC97_RECORD_GAIN		0x1C	/* Record Gain */
#define AC97_RECORD_GAIN_MIC		0x1E	/* Record Gain Mic */
#define AC97_GENERAL_PURPOSE 		0x20	/* General Purpose */
#define AC97_3D_CONTROL			0x22	/* 3D Control */
#define AC97_POWERDOWN_CTRL		0x26	/* Powerdown Ctrl/Stat */
/* range 0x28-0x3a - AUDIO AC'97 2.0 extensions */
#define AC97_EXTENDED_ID		0x28	/* Extended Audio ID */
#define AC97_EXTENDED_STATUS		0x2a	/* Extended Audio Status and Control */
#define AC97_PCM_FRONT_DAC_RATE 	0x2c	/* PCM Front DAC Rate */
#define AC97_PCM_SURR_DAC_RATE		0x2e	/* PCM Surround DAC Rate */
#define AC97_PCM_LFE_DAC_RATE		0x30	/* PCM LFE DAC Rate */
#define AC97_PCM_LR_ADC_RATE		0x32	/* PCM LR ADC Rate */
#define AC97_PCM_MIC_ADC_RATE		0x34	/* PCM MIC ADC Rate */
#define AC97_CENTER_LFE_MASTER		0x36	/* Center + LFE Master Volume */
#define AC97_SURROUND_MASTER		0x38	/* Surround (Rear) Master Volume */
#define AC97_SPDIF			0x3a	/* S/PDIF control */

/*****************************************************************************/

/*
	14-3. Native Audio Bus Master Control Registers
*/

/*
	Next code borrowed from: intel8x0.c
		
	ALSA driver for Intel ICH (i8x0) chipsets
	Copyright (c) 2000 Jaroslav Kysela <perex@suse.cz>
*/

#define ICHREG(x) ICH_REG_##x

#define DEFINE_REGSET(name,base) \
enum { \
	ICH_REG_##name##_BDBAR	= base + 0x0,	/* dword - buffer descriptor list base address */ \
	ICH_REG_##name##_CIV	= base + 0x04,	/* byte - current index value */ \
	ICH_REG_##name##_LVI	= base + 0x05,	/* byte - last valid index */ \
	ICH_REG_##name##_SR	= base + 0x06,	/* byte - status register */ \
	ICH_REG_##name##_PICB	= base + 0x08,	/* word - position in current buffer */ \
	ICH_REG_##name##_PIV	= base + 0x0a,	/* byte - prefetched index value */ \
	ICH_REG_##name##_CR	= base + 0x0b,	/* byte - control register */ \
};

/* Relative addresses */
#define ICH_X_BDBAR	0x00
#define ICH_X_CIV	 0x04
#define ICH_X_LVI	 0x05
#define ICH_X_SR	 0x06
#define ICH_X_PICB	 0x08
#define ICH_X_PIV	 0x0a
#define ICH_X_CR	 0x0b

/* busmaster blocks */
DEFINE_REGSET(OFF, 0);		/* offset */
DEFINE_REGSET(PI,  0x00);	/* PCM in channel  */
DEFINE_REGSET(PO,  0x10);	/* PCM out channel */
DEFINE_REGSET(MC,  0x20);	/* Mic in channel  */

/* ICH4 busmaster blocks */
DEFINE_REGSET(MC2, 0x40);	/* Mic in 2 channel  */
DEFINE_REGSET(PI2, 0x50);	/* PCM in 2 channel  */
DEFINE_REGSET(SP,  0x60);	/* SPDIF out channel */

/* values for each busmaster block */

/* LVI */
#define ICH_REG_LVI_MASK	0x1f

/* SR */
#define ICH_FIFOE		0x10	/* FIFO error */
#define ICH_BCIS		0x08	/* buffer completion interrupt status */
#define ICH_LVBCI		0x04	/* last valid buffer completion interrupt */
#define ICH_CELV		0x02	/* current equals last valid */
#define ICH_DCH			0x01	/* DMA controller halted */

/* PIV */
#define ICH_REG_PIV_MASK	0x1f	/* mask */

/* CR */
#define ICH_IOCE		0x10	/* interrupt on completion enable */
#define ICH_FEIE		0x08	/* fifo error interrupt enable */
#define ICH_LVBIE		0x04	/* last valid buffer interrupt enable */
#define ICH_RESETREGS	0x02	/* reset busmaster registers */
#define ICH_STARTBM		0x01	/* start busmaster operation */


/* global block */
#define ICH_REG_GLOB_CNT	0x2c		/* dword - global control */
#define   ICH_PCM_SPDIF_MASK	0xc0000000	/* s/pdif pcm slot mask (ICH4) */
#define   ICH_PCM_SPDIF_NONE	0x00000000	/* reserved - undefined */
#define   ICH_PCM_SPDIF_78	0x40000000	/* s/pdif pcm on slots 7&8 */
#define   ICH_PCM_SPDIF_69	0x80000000	/* s/pdif pcm on slots 6&9 */
#define   ICH_PCM_SPDIF_1011	0xc0000000	/* s/pdif pcm on slots 10&11 */
#define   ICH_PCM_20BIT		0x00400000	/* 20-bit samples (ICH4) */
#define   ICH_PCM_246_MASK	0x00300000	/* 6 channels (not all chips) */
#define   ICH_PCM_6		0x00200000	/* 6 channels (not all chips) */
#define   ICH_PCM_4		0x00100000	/* 4 channels (not all chips) */
#define   ICH_PCM_2		0x00000000	/* 2 channels (stereo) */
#define   ICH_SIS_PCM_246_MASK	0x000000c0	/* 6 channels (SIS7012) */
#define   ICH_SIS_PCM_6		0x00000080	/* 6 channels (SIS7012) */
#define   ICH_SIS_PCM_4		0x00000040	/* 4 channels (SIS7012) */
#define   ICH_SIS_PCM_2		0x00000000	/* 2 channels (SIS7012) */
#define   ICH_TRIE		0x00000040	/* tertiary resume interrupt enable */
#define   ICH_SRIE		0x00000020	/* secondary resume interrupt enable */
#define   ICH_PRIE		0x00000010	/* primary resume interrupt enable */
#define   ICH_ACLINK		0x00000008	/* AClink shut off */
#define   ICH_AC97WARM		0x00000004	/* AC'97 warm reset */
#define   ICH_AC97COLD		0x00000002	/* AC'97 cold reset */
#define   ICH_GIE		0x00000001	/* GPI interrupt enable */

#define ICH_REG_GLOB_STA	0x30		/* dword - global status */
#define   ICH_TRI		0x20000000	/* ICH4: tertiary (AC_SDIN2) resume interrupt */
#define   ICH_TCR		0x10000000	/* ICH4: tertiary (AC_SDIN2) codec ready */
#define   ICH_BCS		0x08000000	/* ICH4: bit clock stopped */
#define   ICH_SPINT		0x04000000	/* ICH4: S/PDIF interrupt */
#define   ICH_P2INT		0x02000000	/* ICH4: PCM2-In interrupt */
#define   ICH_M2INT		0x01000000	/* ICH4: Mic2-In interrupt */
#define   ICH_SAMPLE_CAP	0x00c00000	/* ICH4: sample capability bits (RO) */
#define   ICH_SAMPLE_16_20	0x00400000	/* ICH4: 16- and 20-bit samples */
#define   ICH_MULTICHAN_CAP	0x00300000	/* ICH4: multi-channel capability bits (RO) */
#define   ICH_SIS_TRI		0x00080000	/* SIS: tertiary resume irq */
#define   ICH_SIS_TCR		0x00040000	/* SIS: tertiary codec ready */
#define   ICH_MD3		0x00020000	/* modem power down semaphore */
#define   ICH_AD3		0x00010000	/* audio power down semaphore */
#define   ICH_RCS		0x00008000	/* read completion status */
#define   ICH_BIT3		0x00004000	/* bit 3 slot 12 */
#define   ICH_BIT2		0x00002000	/* bit 2 slot 12 */
#define   ICH_BIT1		0x00001000	/* bit 1 slot 12 */
#define   ICH_SRI		0x00000800	/* secondary (AC_SDIN1) resume interrupt */
#define   ICH_PRI		0x00000400	/* primary (AC_SDIN0) resume interrupt */
#define   ICH_SCR		0x00000200	/* secondary (AC_SDIN1) codec ready */
#define   ICH_PCR		0x00000100	/* primary (AC_SDIN0) codec ready */
#define   ICH_MCINT		0x00000080	/* MIC capture interrupt */
#define   ICH_POINT		0x00000040	/* playback interrupt */
#define   ICH_PIINT		0x00000020	/* capture interrupt */
#define   ICH_NVSPINT		0x00000010	/* nforce spdif interrupt */
#define   ICH_MOINT		0x00000004	/* modem playback interrupt */
#define   ICH_MIINT		0x00000002	/* modem capture interrupt */
#define   ICH_GSCI		0x00000001	/* GPI status change interrupt */

#define ICH_REG_ACC_SEMA	0x34		/* byte - codec write semaphore */
#define   ICH_CAS		0x01		/* codec access semaphore */

#define ICH_REG_SDM		0x80
#define   ICH_DI2L_MASK		0x000000c0	/* PCM In 2, Mic In 2 data in line */
#define   ICH_DI2L_SHIFT	6
#define   ICH_DI1L_MASK		0x00000030	/* PCM In 1, Mic In 1 data in line */
#define   ICH_DI1L_SHIFT	4
#define   ICH_SE		0x00000008	/* steer enable */
#define   ICH_LDI_MASK		0x00000003	/* last codec read data input */

#define ICH_MAX_FRAGS		32		/* max hw frags */

/* BDL policy bits */
#define ICH_IOC_ENABLE	0x8000
#define ICH_BUP_ENABLE	0x4000
