/*
 * Intel ICH4 Sound Driver for Minix3
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * 
 * History
 * 
 * 2007-05-02	Jozef Hopko		Hardware depended part, modified framework 
 * 
 */

#include "audio_fw.h"
#include "intel8x0.h"
#include "ac97.h"

#define reg(n) dev.base + n

FORWARD _PROTOTYPE( int detect_hw, (void) );  
FORWARD _PROTOTYPE( int disable_int, (int sub_dev) );
FORWARD _PROTOTYPE( int set_stereo, (u32_t stereo, int sub_dev) );
FORWARD _PROTOTYPE( int set_bits, (u32_t nr_of_bits, int sub_dev) );
FORWARD _PROTOTYPE( int set_sample_rate, (u32_t rate, int sub_dev) );
FORWARD _PROTOTYPE( int set_sign, (u32_t val, int sub_dev) );
FORWARD _PROTOTYPE( int get_max_frag_size, (u32_t * val, int *len, int sub_dev) );
FORWARD _PROTOTYPE( int set_frag_size, (u32_t fragment_size, int sub_dev) );
FORWARD _PROTOTYPE( int set_int_cnt, (int sub_dev) );

DEV_STRUCT dev;
u32_t base = 0;
aud_sub_dev_conf_t aud_conf[4];
ICH_DEV	ich_dev[4];

PUBLIC sub_dev_t		sub_dev[4];
PUBLIC special_file_t	special_file[4];
PUBLIC drv_t			drv;


/* ICH Device Count (without mixer device) */
/*
#define SUB_DEV_COUNT 2 
#define CHECK_SUB_DEV(x) if ((x)<SUB_DEV_COUNT) return EINVAL;
*/

/******************** SOUND DRIVER **********************/
u32_t	bus_master_base;
u32_t	mixer_base;

FORWARD _PROTOTYPE( int snd_read_codec_register, (u16_t offs, u16_t *data) );
FORWARD _PROTOTYPE( int snd_write_codec_register, (u16_t offs, u16_t data) );
FORWARD _PROTOTYPE( int snd_init, (void) );
FORWARD _PROTOTYPE( int snd_init_codec, (void) );
FORWARD _PROTOTYPE( int snd_init_chip, (void) );
FORWARD _PROTOTYPE( int snd_power_up_codec, (void) );
FORWARD _PROTOTYPE( int snd_primary_codec_ready, (void) );
FORWARD _PROTOTYPE( int snd_wait_semaphore, (void) );

FORWARD _PROTOTYPE( void micro_delay, (unsigned long usec));

FORWARD _PROTOTYPE( int snd_alloc_bdlist, (int sub_dev));
FORWARD _PROTOTYPE( int snd_free_bdlist, (int sub_dev));

FORWARD _PROTOTYPE( void snd_set_volume, (void) );

#define OK 0
#define ERR -1

#undef dprint
#undef dprint0
#define dprint0 printf
#define dprint (void)		

typedef struct {
	char	name[8];
	u16_t	vendor_id;
	u16_t	device_id;
	u8_t	bus;
	u8_t	dev;
	u8_t	fnc;	
} DEV_INFO;

#define DEV_COUNT 6

/*	Name		Vendor	Device	Bus	  Dev	Fn	*/
DEV_INFO dev_list[DEV_COUNT] = {
	{"ICH",		0x8086,	0x2415,	0x00, 0x1F,	5},
	{"ICH-0",	0x8086,	0x2425,	0x00, 0x1F,	5},
	{"ICH3", 	0x8086,	0x2445,	0x00, 0x1F,	5},
	{"ICH4",	0x8086,	0x24C5,	0x00, 0x1F, 5},
	{"ICH5",	0x8086,	0x24D5,	0x00, 0x1F, 5},
	{"ICH6",	0x8086,	0x266E,	0x00, 0x1E, 2}
};

DEV_INFO	*dev_info = NULL;


/* OK */
PUBLIC int drv_init(void) 
{
	drv.DriverName       = "Intel8x0";
	drv.NrOfSubDevices   = 3;
	drv.NrOfSpecialFiles = 3;
	
	sub_dev[DAC1_CHAN].readable = 0;
	sub_dev[DAC1_CHAN].writable = 1;
	sub_dev[DAC1_CHAN].DmaSize  = 64 * 1024; /* 32 * 1024*/
	sub_dev[DAC1_CHAN].NrOfDmaFragments = 2; 
	sub_dev[DAC1_CHAN].MinFragmentSize  = 1024;
	sub_dev[DAC1_CHAN].NrOfExtraBuffers = 4;
	
			
	sub_dev[ADC1_CHAN].readable = 1;
	sub_dev[ADC1_CHAN].writable = 0;
	sub_dev[ADC1_CHAN].DmaSize  = 32 * 1024;
	sub_dev[ADC1_CHAN].NrOfDmaFragments = 2;
	sub_dev[ADC1_CHAN].MinFragmentSize  = 1024;
	sub_dev[ADC1_CHAN].NrOfExtraBuffers = 4;

	sub_dev[MIXER].writable = 0;
	sub_dev[MIXER].readable = 0;
	
	special_file[0].minor_dev_nr = 0;
	special_file[0].write_chan   = DAC1_CHAN;
	special_file[0].read_chan    = NO_CHANNEL;
	special_file[0].io_ctl       = DAC1_CHAN;

	special_file[1].minor_dev_nr = 1;
	special_file[1].write_chan   = NO_CHANNEL;
	special_file[1].read_chan    = ADC1_CHAN;
	special_file[1].io_ctl       = ADC1_CHAN;

	special_file[2].minor_dev_nr = 2;
	special_file[2].write_chan   = NO_CHANNEL;
	special_file[2].read_chan    = NO_CHANNEL;
	special_file[2].io_ctl       = MIXER;
}

/* OK */
int drv_init_hw (void)
{
    u16_t i, j;

    /* First, detect the hardware */
    if (detect_hw() != OK) {
		return EIO;
    }
    
	/* Init AC97 Codec */
	if (snd_init()!=OK) {
		return EIO;
    }
    
	/* Set volume. Of course we need a nice mixer to do this */
	snd_set_volume();
    
    /* initialize variables for each sub_device */
    for (i = 0; i < drv.NrOfSubDevices; i++) 
	{
      if(i != MIXER) 
	  {
          aud_conf[i].busy          = 0;
          aud_conf[i].stereo        = DEFAULT_STEREO;
          aud_conf[i].sample_rate   = DEFAULT_RATE;
          aud_conf[i].nr_of_bits    = DEFAULT_NR_OF_BITS;
          aud_conf[i].sign          = DEFAULT_SIGNED;
          aud_conf[i].fragment_size = sub_dev[i].DmaSize / sub_dev[i].NrOfDmaFragments;
      }
	}
	
	/* Initialize ich_dev structure */
	ich_dev[DAC1_CHAN].base = ICH_REG_PO_BDBAR;
	ich_dev[DAC1_CHAN].civ = 0;
	ich_dev[DAC1_CHAN].lvi = 0;
	
	ich_dev[ADC1_CHAN].base = ICH_REG_MC_BDBAR;
	ich_dev[ADC1_CHAN].civ = 0;
	ich_dev[ADC1_CHAN].lvi = 0;

    return OK;
}

/* Check for the vendor_id, device_id in the dev_list */
PRIVATE DEV_INFO *check_list(u16_t v_id, u16_t d_id)
{
	u8_t	i;
	for (i=0; i<DEV_COUNT; i++)
	{
		if (v_id == dev_list[i].vendor_id && 
			d_id == dev_list[i].device_id )
			return &dev_list[i];
	}
	return NULL;
}

/* OK */
PRIVATE int detect_hw(void) 
{
	u32_t	r;
	u16_t	v_id, d_id;
	int		devind;

	pci_init();
	/* get first device and then search through the list */
	r = pci_first_dev(&devind, &v_id, &d_id);
	while( r > 0 ) 
	{
		dev_info = check_list(v_id, d_id);
		if (dev_info != NULL)
				break;

		r = pci_next_dev(&devind, &v_id, &d_id);
	}
	
	/* did we find anything? */
	if (dev_info == NULL) 
		return EIO;
	  
	/* fill dev structure */
  	dev.name     = pci_dev_name(v_id, d_id);
   	dev.base     = pci_attr_r32(devind, PCI_BAR) & 0xfffffffe;
  	dev.irq      = pci_attr_r8(devind, PCI_ILR);  
  	dev.revision = pci_attr_r8(devind, 0x08);
 	dev.d_id     = d_id;
  	dev.v_id     = v_id;
  	dev.devind   = devind; /* pci device identifier */
	
	/* DEBUG */ printf("%s: Intel %s chipset detected.\n", drv.DriverName, dev_info->name);
  	
	return OK;
}

/* TODO */
/* ref. by: drv_io_ctl.DSPIORESET */
int drv_reset(void) 
{
  /* make a WARM reset */
  u16_t i;
  
  /* DEBUG */ dprint("drv_reset()\n");
  
  return OK;
}

/* TODO */
int drv_start(int sub_dev, int DmaMode)
{
  u32_t	result = 0;
  /* DEBUG */ dprint("drv_start(sub_dev:%d,dma_mode:%d)\n",sub_dev,DmaMode);
 
  /* Write default values to device in case user failed to configure.
     If user did configure properly, everything is written twice.
     please raise your hand if you object against to this strategy...*/
  result |= set_sample_rate(aud_conf[sub_dev].sample_rate, sub_dev);
  result |= set_stereo(aud_conf[sub_dev].stereo, sub_dev);
  result |= set_bits(aud_conf[sub_dev].nr_of_bits, sub_dev);
  result |= set_sign(aud_conf[sub_dev].sign, sub_dev);
  
  if (result) 
  {
    return EIO;
  }
  
  /* if device currently paused, resume */
  drv_resume(sub_dev);
  
  /* enable interrupts from 'sub device' */
  /*drv_reenable_int(sub_dev);
  
  /* this means GO!!! */
  aud_conf[sub_dev].busy = 1;
  return OK;
}

/* OK */
/* Stop the codec, disable interrupts */
int drv_stop(int sub_dev)
{
  /* DEBUG */ dprint("drv_stop(sub_dev:%d)\n",sub_dev);
  
  aud_conf[sub_dev].busy = 0;

  drv_pause(sub_dev);
  disable_int(sub_dev);

  snd_free_bdlist(sub_dev);
  return OK;
}
/*
void dump_regs()
{
	int i;
	u8_t  val8;
	u16_t val16;
	u32_t val;
	printf("--- AC97 Codec Registers ---\n");
	for (i=0; i<0x7F; i+=2*4)
	{
		snd_read_codec_register(i+0, &val16);
		printf("%02x = %04X   ",i+0, val16);
		snd_read_codec_register(i+2, &val16);
		printf("%02x = %04X   ",i+2, val16);
		snd_read_codec_register(i+4, &val16);
		printf("%02x = %04X   ",i+4, val16);
		snd_read_codec_register(i+6, &val16);
		printf("%02x = %04X   ",i+6, val16);
		snd_read_codec_register(i+8, &val16);
		printf("%02x = %04X \n",i+8, val16);
	}
	
	printf("--- AC97 Controller Registers ---\n");
	val = pci_inl(bus_master_base + ICH_REG_GLOB_CNT);
   	printf("GLOB_CNT: %08X\n",val);	
	val = pci_inl(bus_master_base + ICH_REG_GLOB_STA);
   	printf("GLOB_STA: %08X\n",val);	
	val = pci_inl(bus_master_base + ICH_REG_SDM);
   	printf("SDM: %02X\n",val);	
}
*/

/* OK */
/* all IO-ctl's sent to the upper driver are passed to this function */
int drv_io_ctl(int request, void * val, int * len, int sub_dev) 
{
	int status;
	switch(request) {
		case DSPIORATE:		     status = set_sample_rate(*((u32_t *) val), sub_dev); break;
		
		case DSPIOSTEREO:	     status = set_stereo(*((u32_t *) val), sub_dev); break;
		case DSPIOBITS:	         status = set_bits(*((u32_t *) val), sub_dev); break;
		case DSPIOSIZE:	         status = set_frag_size(*((u32_t *) val), sub_dev); break;
		case DSPIOSIGN:	         status = set_sign(*((u32_t *) val), sub_dev); break;
		case DSPIOMAX:           status = get_max_frag_size(val, len, sub_dev);break;

		case DSPIORESET:         status = drv_reset(); break;
		case AC97READ:           status = snd_read_codec_register (*((u16_t *)val),  ((u16_t *) val+2));break;
		case AC97WRITE:          status = snd_write_codec_register (*((u16_t *)val), *((u16_t *) val+2));break;
		default:                 status = EINVAL; break;
	}
	  
  return OK;
}

/* OK */
int drv_get_irq(char *irq) 
{
	*irq = dev.irq;
	return OK;
}

/* OK */
int drv_get_frag_size(u32_t *frag_size, int sub_dev) 
{
	*frag_size = aud_conf[sub_dev].fragment_size;
    /* DEBUG */ dprint("drv_get_frag_size(frag_size:%d)\n",frag_size);
	return OK;  
}



PRIVATE int reset_bm_regs(int bd_base)
{
	int count = 100;
	int val;
	
	/* Reset registers */
	val = pci_inb(bus_master_base + bd_base + ICH_X_CR); 
  	val|= ICH_RESETREGS;
  	pci_outb(bus_master_base + bd_base + ICH_X_CR, val); 
	
	/* Wait until reset condition is cleared*/
	while (count--)
	{
		if ((pci_inb( bus_master_base + bd_base + ICH_X_CR) & ICH_RESETREGS) == 0)
				goto _reset_ok;
		micro_delay(1000);
	}
	return ERR;
_reset_ok:
	/* We only want interrupts on completition */
	val = pci_inb(bus_master_base + bd_base + ICH_X_CR); 
  	val = ICH_IOCE | ICH_LVBIE /*| ICH_FEIE | ICH_STARTBM*/;
 
    pci_outb(bus_master_base + bd_base + ICH_X_CR, val); 
	return ERR;
}

/*
	This function allocate memory for 32 BD entries, align the pointer to 
	this memory to 8 bytes and umap it to physical address. 
*/
PRIVATE int	snd_alloc_bdlist(int sub_dev)
{
	ICH_DEV	*dev = &ich_dev[sub_dev];
	
	u32_t	*lpListA;	/* Pointer to BDList buffer, not aligned, Allocated */
	BDENTRY	*lpList;	/* Pointer to BDList buffer, 8 bytes align  */
	u32_t 	PhysAddr;	/* Physical address of lpList buffer */
	u32_t	size;
	u32_t	align;
 	int	i;	
	/* Allocate buffer in memory. We need to allocate
	   extra space for later alignment. */
	size = sizeof(BDENTRY)*32 + 64*1024;
  	if (!(lpListA = malloc(size))) {
		printf("snd_alloc_bdlist: malloc failed!\n");
		return ERR;
	}
	
	/* Map virtual address of lpListA buffer to physical address */  	  	
	i = sys_umap(SELF, D, 
               (vir_bytes) lpListA, 
               (phys_bytes) size, 
               &(PhysAddr));
	
	/* Re-aling the buffer to 8 bytes align.*/
	#define dma_bytes_leftB(phys)	((unsigned) (sizeof(int) == 2 ? 0 : 0x10000) - (unsigned) ((phys) & 0xFFFF))
	
	align = dma_bytes_leftB(PhysAddr);
	
	PhysAddr += align;
	lpList = (BDENTRY*)&lpListA[align/4];
	
	/* Update sub_dev structure */
	dev->lpListA  = lpListA; 	
	dev->lpList	  = lpList; 	
	dev->PhysAddr = PhysAddr; 	
	return OK;
}

/*
	This function frees the memory occupied by the BD list.
 */
PRIVATE int snd_free_bdlist(int sub_dev)
{
 	free(ich_dev[sub_dev].lpListA);
	return OK;
}

/* 
 * This function arrange the Buffer Descriptio List. BDENTRY structure
 * represents one entry from BD list. BD list can take up to 32 entries.
 */
int drv_set_dma(u32_t dma, u32_t length, int chan) 
{
	u32_t	bd_base = ich_dev[chan].base;
	BDENTRY	*bdlist;
	
	int i;
	int frag_size = length >> 1;
	
	/* DEBUG */ dprint("drv_set_dma(buff:%X, size:%d, chan:%d) frag_size:%d\n",dma,length,chan,  frag_size);
	
	/* Pause DMA */
	drv_pause(chan);

  	/* Reset Bus Master registers */
	reset_bm_regs(bd_base);  
	
	/* Allocate BD List buffer */
  	snd_alloc_bdlist(chan);
	
	/* Update BD List */
	bdlist = ich_dev[chan].lpList;
	for (i=0; i<32; i+=2)
	{
 		bdlist[i+0].lpBuff   = dma;
 		bdlist[i+0].wPolicy  = ICH_IOC_ENABLE | ICH_BUP_ENABLE;
 		bdlist[i+0].wLength  = (frag_size >> 1);
 		
		bdlist[i+1].lpBuff   = dma + frag_size; 
 		bdlist[i+1].wPolicy  = ICH_IOC_ENABLE | ICH_BUP_ENABLE;
 		bdlist[i+1].wLength  = (frag_size >> 1);
	}
	
	/* Set DBBAR */
	pci_outl(bus_master_base + bd_base, (u32_t)ich_dev[chan].PhysAddr);
	/*pci_outb(bus_master_base + bd_base + 0x4 , 0); /* ? CIV - R/O  */
	
	/* Set Last Valid Index */
	pci_outb(bus_master_base + bd_base + ICH_X_LVI , 0); 
	
	/* MIC: Enable */
	pci_outb(bus_master_base + ICH_REG_SDM, 8);
	
	/* clear interrupts */
 	pci_outw(bus_master_base + bd_base + ICH_X_SR, ICH_FIFOE | ICH_BCIS | ICH_LVBCI); 
	
	/*dump_regs();*/
}

/* TODO */
/* return status of the interrupt summary bit */
int drv_int_sum(void) 
{
	u16_t sum;
	sum = pci_inl(bus_master_base + ICH_REG_GLOB_STA);
	/* DEBUG */ dprint("drv_int_sum(%X)\n",sum & 0xE);
	return sum & (0xE0); /* bit 5,6,7 */
}

/* OK */
/* Check for 'Buffer Completion Interrupt Status' */
/* Return status of interrupt bit of specified channel*/
int drv_int(int sub_dev) 
{
	u16_t sreg = ich_dev[sub_dev].base + ICH_X_SR;
	u16_t status;
	u32_t data;
	
	/* Get interrupt state */
	status  = pci_inw(bus_master_base + sreg);
	/* Clear interrupt */
	pci_outw(bus_master_base + sreg, status); 
	
	/* DEBUG */
	/*	dprint("drv_int(sub_dev:%d,state:%d)\n",sub_dev, status);
	*/	
			
	/* Acknowledge the interrupt */
	data = pci_inl(bus_master_base + ICH_REG_GLOB_STA);
	pci_outl(bus_master_base + ICH_REG_GLOB_STA, data);
	
	/* We need only the interrupt bits */
	return status & (ICH_BCIS | ICH_LVBCI | ICH_FIFOE);
}

/* Confirm and reenable interrupt */
int drv_reenable_int(int chan) 
{
	drv_resume(chan);
	return OK;
}

/* Pause DMA Transfer */
int drv_pause(int sub_dev)
{ 
  u32_t cr_reg = ich_dev[sub_dev].base + ICH_X_CR;
  u8_t	val;
  
  /* DEBUG */ dprint("drv_pause(sub_dev:%d)\n",sub_dev);
	
  /* Pause DMA Transfer */
  val = pci_inb(bus_master_base + cr_reg); 
  val &= ~ICH_STARTBM;
  pci_outb(bus_master_base + cr_reg, val); 
  return OK;
}

/* Resume DMA Transfer */
int drv_resume(int sub_dev)
{
  u32_t cr_reg = ich_dev[sub_dev].base;
  u8_t	val;
  /* DEBUG */ dprint("drv_resume(sub_dev:%d)\n",sub_dev);
  
  /* val = pci_inb(bus_master_base + cr_reg + ICH_X_CIV); /* current index */
  
  val = pci_inb(bus_master_base + cr_reg + ICH_X_CIV); /* last valid */
  val = (val+1) & 31;
  pci_outb(bus_master_base + cr_reg + ICH_X_LVI, val); /* last valid */
  
  /* Resume DMA, enable interrupt on completion (IOCE) */
  
  val = pci_inb(bus_master_base + cr_reg + ICH_X_CR); 
  val|= ICH_IOCE | ICH_LVBIE /*| ICH_FEIE*/ | ICH_STARTBM;
  pci_outb(bus_master_base + cr_reg + ICH_X_CR, val); 
  
  return OK;
}

/* OK */
/* NOTE: 
 * When greater than 16-bit audio is used, then
 * data structures are aligned as 32-bits per sample,
 * with the highest order bits representing the data
 * and the lower bits as don't care */
PRIVATE int set_bits(u32_t nr_of_bits, int sub_dev) 
{
	u32_t val;
	
	if (nr_of_bits!=16 && nr_of_bits!=20)
		return EINVAL;

	/* Microphone is not affected */	
	if (sub_dev == ADC1_CHAN)
		return OK;

	/* Set PCM Out Mode */
	val = pci_inl(bus_master_base + ICH_REG_GLOB_CNT);  
	
	if (nr_of_bits == 16) val &= ~ICH_PCM_20BIT; 
	if (nr_of_bits == 20) val |=  ICH_PCM_20BIT; 
	
	pci_outl(bus_master_base + ICH_REG_GLOB_CNT, val);  
	
	aud_conf[sub_dev].nr_of_bits = nr_of_bits;
	return OK;
}

/* TODO  
 * our device does not support mono/stereo
 * Stereo is always ON */
PRIVATE int set_stereo(u32_t stereo, int sub_dev) 
{
	/* device set stereo ... none */ 
 	aud_conf[sub_dev].stereo = stereo;
	return OK;
}

/* OK */
/* but not used by our driver */
PRIVATE int set_sign(u32_t val, int sub_dev) 
{
	aud_conf[sub_dev].sign = val;
	return OK;
}

/* OK */
PRIVATE int set_frag_size(u32_t fragment_size, int sub_dev_nr) 
{
	if (fragment_size > (sub_dev[sub_dev_nr].DmaSize / sub_dev[sub_dev_nr].NrOfDmaFragments) || fragment_size < sub_dev[sub_dev_nr].MinFragmentSize) {
		return EINVAL;
	}
	aud_conf[sub_dev_nr].fragment_size = fragment_size;
	return OK;
}

/* OK */
/* NOTE: 
 * Default is 48000Hz. Variable sample rate is only available
 * in Variable Rate PCM Audio mode. 
 * Enable VRA bit in Extended Audio Status/Control Register. */
/* u32_t is WRONG TYPE - snd_write_codec_register(u16_t,u16_t) */
PRIVATE int set_sample_rate(u32_t rate, int sub_dev) 
{
	u32_t audio_src_reg; /* Audio Sample Rate Control Register */

	if (rate > MAX_RATE || rate < MIN_RATE)
		return EINVAL;

	/* set the sample rate for the specified channel*/
	switch(sub_dev) {
		case DAC1_CHAN: audio_src_reg = AC97_PCM_FRONT_DAC_RATE;  break;
		case ADC1_CHAN: audio_src_reg = AC97_PCM_MIC_ADC_RATE;    break;
		default: return EINVAL;
	}
	snd_write_codec_register(audio_src_reg, rate);
	aud_conf[sub_dev].sample_rate = rate;
	return OK;
}

/* referenced by drv_start
 * NOT SUPPORTED BY OUR DRIVER */
PRIVATE int set_int_cnt(int chan) 
{
	return OK;
}

/* OK */
PRIVATE int get_max_frag_size(u32_t * val, int * len, int sub_dev_nr) 
{
	*len = sizeof(*val);
	*val = (sub_dev[sub_dev_nr].DmaSize / sub_dev[sub_dev_nr].NrOfDmaFragments);
	return OK;
}

/* OK */
/* ref. by: drv_pause, drv_stop */
PRIVATE int disable_int(int chan) 
{
	u32_t reg = ich_dev[chan].base + ICH_X_CR;

  	/* DEBUG */ dprint("disable_int(sub_dev:%d)\n",chan);
 	 
	/* disable interrupts */
	pci_outb(bus_master_base + reg, 0x00);
}



/*===========================================================================*
 * START                          SOUND DRIVER                               *
 *===========================================================================*/
#define MICROS_TO_TICKS(m) (((m)*HZ/1000000)+1)
#define MILLIS_TO_TICKS(m) (((m)*HZ/1000   )+1)

/* This function wait for usec micro seconds */
PRIVATE void micro_delay(unsigned long usec)
{
	tickdelay(MICROS_TO_TICKS(usec));
}

/* This function wait for usec milli seconds */
PRIVATE void milli_delay(unsigned long usec)
{
	tickdelay(MILLIS_TO_TICKS(usec));
}

/* Wait for the AC97 Codec semaphore. If the CAS bit is
   cleared there is no codec access in progress, so we
   can access the code. */
PRIVATE int snd_wait_semaphore()
{
	int count = 100;
	while (count--)
	{
		if ((pci_inb( bus_master_base + ICH_REG_ACC_SEMA ) & ICH_CAS) == 0)
				return OK;
		/* The minimum latency of codec is 40 micro seconds */
		micro_delay(40);
	}
	return ERR;
}

/* Reads a AC97 register. */
PRIVATE int snd_read_codec_register(u16_t offs, u16_t *data)
{
	int status;
	/* Wait for semaphore */
	if (snd_wait_semaphore()!=OK)
			return ERR;

	/* Read the data */	
	*data = pci_inw(mixer_base+offs);

	/* Check if read was sucessful. The RCS(Read Completion Status)
	   bit must be cleared. Otherwise there was an error, and
	   we need to clear the RCS bit ourself. */	
	status = pci_inl(bus_master_base+ICH_REG_GLOB_STA);
	if (status & ICH_RCS)
	{
		/* Clear timeout bit */
		pci_outl(bus_master_base + ICH_REG_GLOB_STA, status); 
		return ERR;
	}

	return OK;
}

/* Writes to a AC97 register. */
PRIVATE int snd_write_codec_register(u16_t offs, u16_t data)
{
	int status;
	/* Wait for semaphore */
	if (snd_wait_semaphore()!=OK)
			return ERR;

	/* Write the data */	
	pci_outw(mixer_base+offs, data);

	return OK;
}

/* Check whether the primary codec is ready. This may take awhile. */
PRIVATE int snd_primary_codec_ready()
{
	u32_t reg;
	int count = 1000;
	
	/* Put logic to right state */
	reg = pci_inl(bus_master_base+ICH_REG_GLOB_STA);
	reg = reg & (ICH_RCS | ICH_MCINT | ICH_POINT | ICH_PIINT);
	pci_outl(bus_master_base+ICH_REG_GLOB_STA, reg);
	
	/* Enable AC link and perform reset */	
	reg = pci_inl(bus_master_base+ICH_REG_GLOB_CNT);
	
    reg&= ~(ICH_ACLINK | ICH_PRIE | ICH_PCM_246_MASK);
	reg|= (reg & ICH_AC97COLD) == 0 ? ICH_AC97COLD : ICH_AC97WARM;
	
	/*reg = (reg | ICH_AC97COLD) & ~(ICH_ACLINK | ICH_PRIE);
	*/
	pci_outl(bus_master_base+ICH_REG_GLOB_CNT, reg);
	
	/* Wait for reset */
	while (count--)
	{
		if ((pci_inl(bus_master_base+ICH_REG_GLOB_CNT) & ICH_AC97WARM) == 0)
			goto _ok;
		milli_delay(1);
	}
	
	return ERR;

_ok:
	count = 1000;
	/* Wait for codec to be ready */
	while (count--)
	{
		if (pci_inl(bus_master_base+ICH_REG_GLOB_STA) & ICH_PCR)
			return OK;
		milli_delay(1);
	}
	
	return ERR;
}

/* Set the codec to the D0 power state - all codec parts ON. */
PRIVATE int snd_power_up_codec()
{
	u16_t reg;
	int count=1000;
	
	/* Power up codec */
	if (snd_write_codec_register(AC97_POWERDOWN_CTRL, 0x00)!=OK)
		dprint("ERR: power_up.write_codec_reg POWERDN failed.");
	
	/* Wait for codec to be powered up */
	while (count--)
	{
		snd_read_codec_register(AC97_POWERDOWN_CTRL,&reg);
		if (reg & 0xF == 0xF)
			return OK;
		
		milli_delay(1);
	}
	
	return ERR;
}

/* Initialize the AC7 Codec */
PRIVATE int snd_init_codec()
{
	int status;
	/* Check if the AC link to primary codec is ready */
	dprint("Primary codec ready\n");
	status = snd_primary_codec_ready();
	if (status == OK)
	{
		/* Reset codec */
		dprint("Reset codec\n");
		snd_write_codec_register(AC97_RESET, 0x00);

		/* Power up */
		dprint("Power up codec\n");
		status = snd_power_up_codec();
	}		
	return status;
}
/* Initialize the ICH chip registers */
PRIVATE int snd_init_chip()
{
	int i;
	int count;
	/* Disable Interrupts */
	for (i=0; i<3; i++)
		pci_outb(bus_master_base + i*0x10 + ICH_X_CR, 0); 
	
    /* Reset Channels */	   	
	for (i=0; i<3; i++)
		pci_outb(bus_master_base + i*0x10 + ICH_X_CR, ICH_RESETREGS);
	
    /* Wait for reset */
	for (i=0; i<3; i++)
	{
	   count = 100;
	   while (--count)
	   {
    		if ((pci_inb(bus_master_base + i*0x10 + ICH_X_CR) & ICH_RESETREGS) == 0)
    		  break;
	       milli_delay(1);
	   }
	   if (count ==0)
	       dprint("Channel %i reset failed.\n", i);
	   
    }
} 	

/* Initialize AC97 and ICH chip. */
PRIVATE int snd_init()
{
	/*	Read Mixer and Bus Master base address 
	 	from PCI configuration space	 */
	u8_t bus_ = dev_info->bus;
	u8_t dev_ = dev_info->dev;
	u8_t fnc_ = dev_info->fnc;
	

	
/*	printf("B:D:F : %d %d %d.\n",bus_, dev_, fnc_ ); */
	mixer_base = PCII_RREG32_(bus_, dev_, fnc_, 0x10) & 0xFFFFFFF8;
  
	bus_master_base = PCII_RREG32_(bus_, dev_, fnc_, 0x14) & 0xFFFFFFF8;

	/* DEBUG */	
/*	dprint("Audio Bus Mastering Base (NABMBAR): %02X\n", bus_master_base);
	dprint("Audio Mixer Base (NAMBAR): %02X\n", mixer_base);
/* */	
	/* Initialize the hadware */
	if (snd_init_codec()!=OK)
	{
		printf("ERROR: codec init failed.");
		return ERR;
	}
/*  
	if (snd_primary_codec_ready()!=OK)
	{
		printf("ERROR: codec not ready.");
		return ERR;
	}
*/
	if (snd_init_chip()!=OK)
	{
		printf("ERROR: chip init failed.");
		return ERR;
	}
	
	pci_outb(bus_master_base + ICH_REG_SDM, ICH_SE /*| (1<<ICH_DI1L_SHIFT)*/);
	
	return OK;	
}

PRIVATE void snd_set_volume(void) 
{
	/* Set Nice Volume */
	snd_write_codec_register(AC97_MASTER_VOLUME,	0x0000);		
	snd_write_codec_register(AC97_AUX_OUT_VOLUME,	0x0000);		
	snd_write_codec_register(AC97_MONO_VOLUME,		0x0000);		
	snd_write_codec_register(AC97_MASTER_TONE,		0x0F0F);		
	snd_write_codec_register(AC97_PC_BEEP_VOLUME,	0x0000);		
	snd_write_codec_register(AC97_PHONE_VOLUME,		0x8008);		
	snd_write_codec_register(AC97_MIC_VOLUME,		0x8008);		
	snd_write_codec_register(AC97_LINE_IN_VOLUME,	0x0808);		
	snd_write_codec_register(AC97_CD_VOLUME,		0x0808);		
	snd_write_codec_register(AC97_VIDEO_VOLUME,		0x0808);		
	snd_write_codec_register(AC97_AUX_IN_VOLUME,	0x0808);		
	snd_write_codec_register(AC97_PCM_OUT_VOLUME,	0x0808);		
	snd_write_codec_register(AC97_RECORD_SELECT,	0x0404);		
	snd_write_codec_register(AC97_RECORD_GAIN,		0x0000);		
	snd_write_codec_register(AC97_RECORD_GAIN_MIC,	0x0000);		
	snd_write_codec_register(AC97_GENERAL_PURPOSE,	0x0000);		
	snd_write_codec_register(AC97_3D_CONTROL,		0x0000);		

	snd_write_codec_register(AC97_EXTENDED_STATUS,	0x4001);		

	snd_write_codec_register(AC97_CENTER_LFE_MASTER,0x0000);		
	snd_write_codec_register(AC97_SURROUND_MASTER,	0x0000);		
	
	/*MUTE PC BEEP*/
	snd_write_codec_register(AC97_PC_BEEP_VOLUME,	0x8000);		
    
	/* Enable 3D Control */	
/*	snd_write_codec_register(AC97_GENERAL_PURPOSE, bit(13));		

	/* Set Enhancement Depth Control */
/*	snd_write_codec_register(AC97_3D_CONTROL, 0xF); /* maximum */		
	
/*	snd_write_codec_register(AC97_MIC_VOLUME, 0x40); /* 0x40 Mic ON,0x7 0FF */
	snd_write_codec_register(AC97_RECORD_SELECT,	0x0000);		
}
/*===========================================================================*
 * END                         SOUND DRIVER                                  *
 *===========================================================================*/

