#ifndef INTEL8X0_H
#define INTEL8X0_H

#include <sys/types.h>
#include <sys/types.h>
#include "../drivers.h"
#include "../pci/pci.h"
#include "../pci/pci_intel.h"
#include <ibm/pci.h>
#include <sys/ioc_sound.h>

/* subdevices */
#define DAC1_CHAN 0		/* Digital-Analog Convertor */
#define ADC1_CHAN 1		/* Analog-Digital Convertor */
#define MIXER 2			/* Mixer */

/* set your vendor and device ID's here */
#define VENDOR_ID 0x8086
#define DEVICE_ID 0x24C5

typedef struct {
  u32_t stereo;
  u16_t sample_rate; 
  u32_t nr_of_bits;
  u32_t sign;
  u32_t busy;
  u32_t fragment_size;
} aud_sub_dev_conf_t;


/* Some defaults for the aud_sub_dev_conf_t*/
#define DEFAULT_RATE		48000     /* Sample rate */
#define DEFAULT_NR_OF_BITS	16	      /* Nr. of bits per sample per channel*/
#define DEFAULT_SIGNED		0          /* 0 = unsigned, 1 = signed */
#define DEFAULT_STEREO		1     	   /* 0 = mono, 1 = stereo */
#define MAX_RATE			48000      /* Max sample speed in KHz */
#define MIN_RATE			8000       /* Min sample speed in KHz */


typedef struct DEVSTRUCT {
    char*     name;
    u16_t     v_id;    /* vendor id */
    u16_t     d_id;    /* device id */
    u32_t     devind;  /* minix pci device id, for pci configuration space */
    u32_t     base;    /* changed to 32 bits */
    char      irq; 
    char      revision;/* version of the device */
} DEV_STRUCT;

#define bit(n) 1UL << n

/**********************************/

/* Buffer Descriptor Entry */
typedef struct
{
	u32_t	lpBuff;		/* Buffer Pointer */
	u16_t	wLength;	/* Buffer Length */
	u16_t	wPolicy;	/* Policy Bits */
} BDENTRY;


typedef struct {
	u32_t	base;		/* Buffer Descriptor Base Address used for PCI I/O access*/
	u16_t	civ;		/* Current Index Value */
	u16_t	lvi;		/* Last Index Value */

	u32_t	*lpListA;	/* Pointer to BDList buffer, not aligned, Allocated */
	BDENTRY	*lpList;	/* Pointer to BDList buffer, 8 bytes align  */
	u32_t 	PhysAddr;	/* Physical address of lpList buffer - Buffer Descriptor List */
} ICH_DEV;

#endif /* INTEL8X0 */
