/*   
 *  mixer
 *
 *  Michel R. Prevenier.
 */

#include <sys/types.h>
#include <errno.h>
#include <curses.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <minix/sound.h>

#define CURS_CTRL	'\033'
#define ESCAPE		27
#define UP   		'A'
#define DOWN		'B'
#define LEFT		'D'
#define RIGHT		'C'
#define SPACE		' '


_PROTOTYPE ( int main, (int arg, char **argv));
_PROTOTYPE ( void setup_screen, (void));
_PROTOTYPE ( void terminate, (int s));

WINDOW *main_win;
int old_stdin;
int fd;
char name[9];
char *file_name;
struct volume_level levels[9];
struct inout_ctrl inputs_left[9];
struct inout_ctrl inputs_right[9];
struct inout_ctrl outputs[9];


void usage()
{
  fprintf(stderr, "Usage: mixer [-r]\n");
  exit(-1);
}


void terminate(s)
int s;
{
  /* Restore terminal parameters and exit */

  (void) fcntl(0,F_SETFL,old_stdin);
  move(23, 0);			
  refresh();			
  resetty();
  endwin();			
  exit(1);		
}



int main(argc, argv)
int argc;
char **argv;

{
  int i;
  char *home_ptr;
  int fd2;

  /* Initialize windows. */
  (void) initscr();
  signal(SIGINT, terminate);
  old_stdin = fcntl(0,F_GETFL);
  cbreak();
  noecho();
  main_win = newwin(80,23,0,0);
  scrollok(main_win, FALSE);

  /* Set up the user screen and handle user input */
  setup_screen();
}

#define draw_char(x,y,c) mvwaddch(main_win, y, x, c);
#define draw_text mvwprintw

void show_inputs(x,y)
int x;
int y;
{
  /* Show the input settings */

  int i;

  mvwprintw(main_win,y,x,  "             Rec-In  ");
  mvwprintw(main_win,y+1,x,"          left    right");
  mvwprintw(main_win,y+2,x,"          l   r   l   r");
  wrefresh(main_win);
}

void draw_border(int x, int y, int w, int h)
{
	/* Corner:         UL   UR   DL   DR */
	char corner[4] = {218, 191, 192, 217};
	/* Line:        Horz Vert */
	char line[2] = { 179, 196 };

	int i;
	/* Draw corners */
  	draw_char(x+0, y+0, corner[0]);
  	draw_char(x+w, y+0, corner[1]);
  	draw_char(x+0, y+h, corner[2]);
  	draw_char(x+w, y+h, corner[3]);
	/* Draw lines */
	for (i=1; i<w; i++)
	{
  		draw_char(x+i, y+0, line[1]);
  		draw_char(x+i, y+h, line[1]);
	}
	
	for (i=1; i<h; i++)
	{
  		draw_char(x+0, y+i, line[0]);
  		draw_char(x+w, y+i, line[0]);
	}
}

void draw_control(int x, int y)
{
	char box[2] = {/*177*/' ',219};
	int i,p=4;
	draw_border(x, y, 3, 17);
	
	for (i=0; i<16; i++)
	{
  		draw_char(x+1, y+i+1, box[i<p?0:1]);
  		draw_char(x+2, y+i+1, box[i<p?0:1]);
	}
			
  	draw_text(main_win,y+18,x-1,"Master");
}

void setup_screen()
{
  int i;

  wclear(main_win);
  mvwprintw(main_win,0,23,"------- Mixer Controls -------");
  wrefresh(main_win);
 
  /*show_inputs(41,16);
  */
  /*
  wattrset(main_win, (i)<<8 );
  */
/*  mvwprintw(main_win,1,1,"%X",i);
  wbox(main_win, 2,2, 10, 20, 'A', 'B');
  mvwprintw(main_win,2,4,"%c",186);
*/
  draw_control(5,2);
  wrefresh(main_win);
}
