/*   
 *  playwave.c
 *
 *  Play sound files in wave format. Only MicroSoft PCM is supported. 
 *
 *  Michel R. Prevenier.
 */

#include <sys/types.h>
#include <errno.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <minix/sound.h>

_PROTOTYPE( int main, (int argc, char **argv));
_PROTOTYPE( void usage, (void));

/******* Wave format definitions *********/

#define RIFF_ID		0x46464952
#define WAVE_ID1	0x45564157
#define WAVE_ID2	0x20746D66
#define DATA_ID		0x61746164
#define MS_PCM_FORMAT	0x0001
	
#define WORD	short  
#define DWORD   unsigned long

struct RIFF_fields
{
  DWORD RIFF_id;
  DWORD RIFF_len;
  DWORD WAVE_id1;
  DWORD WAVE_id2;
  DWORD data_ptr;
} r_fields; 

struct common_fields
{
  WORD  FormatTag;
  WORD  Channels;
  DWORD SamplesPerSec;
  DWORD AvgBytesPerSec;
  WORD  BlockAlign;
} c_fields;

struct specific_fields
{
  WORD BitsPerSample;
} s_fields;

DWORD data_id;
DWORD data_len;

/******** End of wave definitions *********/


void usage()
{
  fprintf(stderr, "Usage: playwav [-i] file\n");
  exit(-1);
}

/*
 	MP3 Decoder
 */

#include "mpg123.h"
#include "mpglib.h"

#define BUFFER_MAX (16*1024) 			/* 16384 */ 
#define BUFFER_MAX2 (BUFFER_MAX>>1) /*  8192 */

struct	mpstr mp;

void mp3_init()
{
	InitMP3(&mp);
}

void mp3_close()
{
}

/*
 */

unsigned char	*buffer; 
unsigned int	buffer_size;
unsigned int	fragment_size;
  
int i, audio, file;

#define FRAGS 6 

#include <minix/sysutil.h>
#include <sys/times.h>
clock_t clk;
struct tms tm;
struct tms tmn;
void start_watch()
{
	clk = times(&tm);	
/*	getuptime(&clk);
*/
}
void stop_watch(char *str)
{
	clock_t now;
	now = times(&tmn);	
/*	getuptime(&now);*/
	printf("%s: %.2f sec\n",str, (now - clk)/55.);
}

int mp3_write_buffer(char *buff, int size)
{
	int bytes_needed = fragment_size*FRAGS - buffer_size;
	int i;
	if (size >= bytes_needed) /* ak mame rovnako alebo viac bajtov ako potrebujeme */
	{
		/* doplnime buffer */
		memcpy(&buffer[buffer_size], buff, bytes_needed);
			
		/*printf("Playing. Sample size:%ld\n", size);
		*/
		printf("Playing %d buffers (%ld bytes)\n",FRAGS, fragment_size*FRAGS);
		/* play */
		
		for (i=0;i<FRAGS;i++)
		{
			write(audio, &buffer[fragment_size*i], fragment_size);
		}
		/* odstrineme ulozene data */
		buff = &buff[bytes_needed];
		size = size - bytes_needed;
		buffer_size = 0; /* vycistime buffer */	
	}
	
	if (size < bytes_needed) /* ak mame menej bajtov ako potrebujeme */
	{
		memcpy(&buffer[buffer_size], buff, size);
		buffer_size += size; 
	}	
}

int mp3_start_decode(void)
{
	int size;
	static char out[BUFFER_MAX2];
	static char buf[BUFFER_MAX]; 
	int len,ret;
	
	int first_time = 1;
	int freq;
	/*
 	if ((buf = malloc(BUFFER_MAX)) == 0)
  	{
    	fprintf(stderr, "Cannot allocate buffer\n");
    	exit(-1);
  	}
 	if ((out = malloc(BUFFER_MAX2)) == 0)
  	{
    	fprintf(stderr, "Cannot allocate buffer\n");
    	exit(-1);
  	}
  */
	buffer_size = 0;
	printf("Start MP3...\n");
	while(1) {
		len = read(file,buf,BUFFER_MAX);
		if(len <= 0)
			break;
		start_watch();
		ret = decodeMP3(&mp,buf,len,out,BUFFER_MAX2,&size);
		if (first_time == 1)
		{
			freq = freqs[mp.fr.sampling_frequency];
			printf("Sample rate: %i Hz\n",freq);	
  			ioctl(audio, DSPIORATE, &freq);
			first_time = 0;
		}
		while(ret == MP3_OK)
		{
			mp3_write_buffer(out,size);
			ret = decodeMP3(&mp,NULL,0,out,BUFFER_MAX2,&size);
		}
		stop_watch("Decoding");
	}
/*
	free(out);
	free(buf);
*/}

/*#define PCM_DECODE
*/
int pcm_start_play(void)
{
	int size;
	int len,ret;
	static char buf[8192]; 
	
	buffer_size = 0;
	printf("Start PCM...\n");
	while(1) {
		len = read(0,buf,8192);
		if(len <= 0)
			break;
		mp3_write_buffer(buf,len);
	}
}


int main ( int argc, char *argv[] )
{
/*  int i, audio, file; */
  char *file_name;
  unsigned int sign;
/*  unsigned int fragment_size;
  */
  unsigned int channels;
  unsigned int bits;
  long data_pos;  
  int showinfo = 0;
  int data_len2;
  int size;
  /* Check Parameters */
  if (argc > 2)
  {
    if (strncmp(argv[1], "-i", 2) == 0)
    {
      showinfo = 1;
      file_name = argv[2];
    }
    else
      usage();
  }
  else file_name = argv[1];

  /* Open DSP */
  if ((audio = open("/dev/audio", O_RDWR)) < 0) 
  {
    printf("Cannot open /dev/audio\n");
    exit(-1);
  }

  /* Get maximum fragment size and try to allocate a buffer */
  ioctl(audio, DSPIOMAX, &fragment_size);
  if ((buffer = malloc(fragment_size*FRAGS)) == (unsigned char *)0)
  {
    fprintf(stderr, "Cannot allocate buffer\n");
    exit(-1);
  } 
  ioctl(audio, DSPIOSIZE, &fragment_size);

  goto jump1;
  
  /* Open wav file */
  if((file = open(file_name, O_RDONLY)) < 0)
  {
    printf("Cannot open %s\n", file_name);
    exit(-1);
  }
  
  /* Check for valid wave format */
  read(file, &r_fields, 20);
  if(r_fields.RIFF_id != RIFF_ID)
  {
      printf("%s not in RIFF format\n", file_name);
      exit(1);
  }
  if(r_fields.WAVE_id1 != WAVE_ID1 || r_fields.WAVE_id2 != WAVE_ID2)
  {
      printf("%s not in WAVE format\n", file_name);
      exit(1);
  }

  /* Store data_chunk position */
  data_pos = lseek(file, 0L, 1) + r_fields.data_ptr;

  /* Read the common and specific fields */
  read(file, &c_fields, 14);
  read(file, &s_fields, 2);

  /* Check for valid wave format, we can only play MicroSoft PCM */
  if(c_fields.FormatTag != MS_PCM_FORMAT)
  {
    printf("%s not in MicroSoft PCM format\n", file_name);
    exit(1);
  }

  /* Set DSP parameters */
  channels = c_fields.Channels;
  channels--;
  bits = s_fields.BitsPerSample;
  ioctl(audio, DSPIOSTEREO, &channels); 
  ioctl(audio, DSPIORATE, &c_fields.SamplesPerSec);
  ioctl(audio, DSPIOBITS, &bits); 
  sign = (bits == 16 ? 1 : 0);
  ioctl(audio, DSPIOSIGN, &sign); 

  /* Goto data chunk */
  lseek(file, data_pos, SEEK_SET);

  /* Check for valid data chunk */
  read(file, &data_id, sizeof(data_id));
  if(data_id != DATA_ID)
  {
    printf("Invalid data chunk\n");
    exit(1);
  }

  /* Get length of data */
  read(file, &data_len, sizeof(data_len));

  if (showinfo)
  {
    printf("\nBits per sample   : %d \n", s_fields.BitsPerSample);
    printf("Stereo            : %s \n", (c_fields.Channels == 1 ? "yes" : "no"));
    printf("Samples per second: %ld \n", c_fields.SamplesPerSec); 
    printf("Average bytes/sec : %ld \n", c_fields.AvgBytesPerSec);
    printf("Block alignment   : %d \n", c_fields.BlockAlign);
    printf("Datalength (bytes): %ld \n\n", data_len);
    
	printf("Fragment size     : %ld \n\n", fragment_size);
  }
    
  /* Play data */
  showinfo = -30;
  
  data_len2 = data_len;
 jump1:
  channels=44100;
  channels=22050;
  ioctl(audio, DSPIORATE, &channels);
  data_len = 0;
#ifdef PCM_DECODE 
  pcm_start_play();
#else
  /* Open wav file */
  if((file = open(file_name, O_RDONLY)) < 0)
  {
    printf("Cannot open %s\n", file_name);
    exit(-1);
  }
  
  mp3_init();
  mp3_start_decode();
  mp3_close();
  
  close(file);
  close(audio);
#endif
  return 0;
  while(1)
  {
	data_len = 0;
	data_len2 = fragment_size;
	while ((size = read(0, &buffer[data_len], data_len2))!=0)
	{/*
		printf("Playwave: size: %ld, data_len %ld\n", size, data_len);
	*/
		if (size == 0)
			return 0;
		
		data_len += size;
		
		if (fragment_size - data_len < data_len2)
				data_len2 = fragment_size - data_len;
		
		if (data_len == fragment_size)
			write(audio, buffer, fragment_size);
	}/*
	if (size < fragment_size) 
    { 
      for (i = size; i< fragment_size; i++) 
        buffer[i] = buffer[(int)size-1];
    }
    	
	write(audio, buffer, fragment_size);
	data_len += size;
	printf("Playwave: %ld bytes played %ld\n", data_len, size);
*/
  }
  printf("DONE\n");
}
